@@ -1,5 +1,5 @@
 package Biber::Section;
-use v5.16;
+use v5.24;
 use strict;
 use warnings;
 
@@ -34,12 +34,12 @@ sub new {
   $self->{everykey_lc} = {};
   $self->{bcfkeycache} = {};
   $self->{labelcache_v} = {};
-  $self->{sortcache} = [];
   $self->{dkeys} = {};
   $self->{keytods} = {};
   $self->{orig_order_citekeys} = [];
   $self->{undef_citekeys} = [];
   $self->{citekey_alias} = {};
+  $self->{static_keys} = {};
   return $self;
 }
 
@@ -51,7 +51,6 @@ sub new {
 
 sub reset_caches {
   my $self = shift;
-  $self->{sortcache} = [];
   $self->{labelcache_l} = {};
   $self->{labelcache_v} = {};
   $self->{bcfkeycache} = {};
@@ -295,7 +294,7 @@ sub del_bibentries {
 sub set_citekeys {
   my $self = shift;
   my $keys = shift;
-  map { $self->{citekeys_h}{$_} = 1} @$keys;
+  map { $self->{citekeys_h}{$_} = 1} $keys->@*;
   $self->{citekeys} = $keys;
   return;
 }
@@ -313,7 +312,6 @@ sub set_orig_order_citekeys {
   return;
 }
 
-
 =head2 get_citekeys
 
     Gets the citekeys of a Biber::Section object
@@ -323,7 +321,7 @@ sub set_orig_order_citekeys {
 
 sub get_citekeys {
   my $self = shift;
-  return @{$self->{citekeys}};
+  return $self->{citekeys}->@*;
 }
 
 =head2 get_static_citekeys
@@ -339,6 +337,17 @@ sub get_static_citekeys {
   return reduce_array($self->{citekeys}, $self->dynamic_set_keys);
 }
 
+=head2 has_cited_citekey
+
+    Returns true when $key was one of the actually cited keys in the section
+
+=cut
+
+sub has_cited_citekey {
+  my $self = shift;
+  my $key = shift;
+  return $self->{citekeys_h}{$key} ? 1 : 0;
+}
 
 =head2 add_undef_citekey
 
@@ -351,7 +360,7 @@ sub get_static_citekeys {
 sub add_undef_citekey {
   my $self = shift;
   my $key = shift;
-  push @{$self->{undef_citekeys}}, $key;
+  push $self->{undef_citekeys}->@*, $key;
   return;
 }
 
@@ -364,7 +373,7 @@ sub add_undef_citekey {
 
 sub get_undef_citekeys {
   my $self = shift;
-  return @{$self->{undef_citekeys}};
+  return $self->{undef_citekeys}->@*;
 }
 
 =head2 get_orig_order_citekeys
@@ -378,7 +387,7 @@ sub get_undef_citekeys {
 
 sub get_orig_order_citekeys {
   my $self = shift;
-  return @{$self->{orig_order_citekeys}};
+  return $self->{orig_order_citekeys}->@*;
 }
 
 =head2 has_citekey
@@ -405,8 +414,8 @@ sub del_citekey {
   my $self = shift;
   my $key = shift;
   return unless $self->has_citekey($key);
-  $self->{citekeys}            = [ grep {$_ ne $key} @{$self->{citekeys}} ];
-  $self->{orig_order_citekeys} = [ grep {$_ ne $key} @{$self->{orig_order_citekeys}} ];
+  $self->{citekeys}            = [ grep {$_ ne $key} $self->{citekeys}->@* ];
+  $self->{orig_order_citekeys} = [ grep {$_ ne $key} $self->{orig_order_citekeys}->@* ];
   delete $self->{citekeys_h}{$key};
   return;
 }
@@ -437,13 +446,12 @@ sub add_citekeys {
   foreach my $key (@keys) {
     next if $self->has_citekey($key);
     $self->{citekeys_h}{$key} = 1;
-    push @{$self->{citekeys}}, $key;
-    push @{$self->{orig_order_citekeys}}, $key;
+    push $self->{citekeys}->@*, $key;
+    push $self->{orig_order_citekeys}->@*, $key;
   }
   return;
 }
 
-
 =head2 set_citekey_alias
 
     Set citekey alias information
@@ -482,7 +490,6 @@ sub del_citekey_alias {
   return;
 }
 
-
 =head2 get_citekey_aliases
 
     Get a list of all citekey aliases for the section
@@ -491,10 +498,9 @@ sub del_citekey_alias {
 
 sub get_citekey_aliases {
   my $self = shift;
-  return ( keys %{$self->{citekey_alias}} );
+  return ( keys $self->{citekey_alias}->%* );
 }
 
-
 =head2 set_labelcache_v
 
     Sets the variable label disambiguation cache for a field
@@ -581,7 +587,7 @@ sub get_dynamic_set {
   my $self = shift;
   my $dkey = shift;
   if (my $set_members = $self->{dkeys}{$dkey}) {
-    return @$set_members;
+    return $set_members->@*;
   }
   else {
     return ();
@@ -596,7 +602,7 @@ sub get_dynamic_set {
 
 sub dynamic_set_keys {
   my $self = shift;
-  return [keys %{$self->{dkeys}}];
+  return [keys $self->{dkeys}->%*];
 }
 
 =head2 has_dynamic_sets
@@ -620,7 +626,7 @@ sub has_dynamic_sets {
 sub add_datasource {
   my $self = shift;
   my $source = shift;
-  push @{$self->{datasources}}, $source;
+  push $self->{datasources}->@*, $source;
   return;
 }
 
@@ -655,34 +661,6 @@ sub get_datasources {
 }
 
 
-=head2 add_sort_cache
-
-    Adds a scheme/keys pair to the sort cache:
-    [$scheme, $keys, $sortinitdata, $extraalphadata, $extrayeardata ]
-
-=cut
-
-sub add_sort_cache {
-  my $self = shift;
-  my $cacheitem = shift;
-  push @{$self->{sortcache}}, $cacheitem;
-  return;
-}
-
-
-=head2 get_sort_cache
-
-    Retrieves the sort cache
-
-=cut
-
-sub get_sort_cache {
-  my $self = shift;
-  return $self->{sortcache};
-}
-
-
-
 =head2 number
 
     Gets the section number of a Biber::Section object
@@ -710,7 +688,7 @@ L<https://github.com/plk/biber/issues>.
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2009-2014 François Charette and Philip Kime, all rights reserved.
+Copyright 2009-2018 François Charette and Philip Kime, all rights reserved.
 
 This module is free software.  You can redistribute it and/or
 modify it under the terms of the Artistic License 2.0.
